IEInterruptAnnounce = LibStub("AceAddon-3.0"):NewAddon("IEInterruptAnnounce", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("IEInterruptAnnounce")

local inRaid
local grouped
local inBG
local inArena
local cooldownReadyTime = GetTime()
local interruptTimestamp = nil
local interruptVariables = nil

local defaults = {
    profile = {
		solo = {
			say = false,
			whisper = false,
			channel = false,
		},
		party = {
			say = false,
			party = true,
			whisper = false,
			channel = false,
		},
		raid = {
			say = false,
			party = true,
			raid = false,
			raidwarning = false,
			whisper = false,
			channel = true,
		},
		whisperName = "",
		channelName = "iekickers",
		advanced = {
			debug = false,
			discoverInterruptMode = false,
			chatThrottleMillis = 10,
			interruptDelayMillis = 10,
			msgFormat = "%s: %s (%s)",
			wastedText = "<Used>",
		},
		spellList = { -- 2 = announce, 1 = announce but not wasted messages, 0 = do not announce
			-- Blood Elf
			[50613] = 1, -- Arcane Torrent (Runic)
			[28730] = 1, -- Arcane Torrent (Mana)
			[25046] = 1, -- Arcane Torrent (Energy)
			-- Death Knight
			[47528] = 2, -- Mind Freeze
			[47476] = 2, -- Strangulate
			-- Druid
			[5211] = 2, -- Bash R1
			[6798] = 2, -- Bash R2
			[8983] = 2, -- Bash R3
			[19675] = 2, -- Feral Charge (Bear)
			-- Hunter
			[34490] = 2, -- Silencing Shot
			-- Mage
			[2139] = 2, -- Counterspell
			-- Paladin
			[853] = 1, -- Hammer of Justice R1
			[5588] = 1, -- Hammer of Justice R2
			[5589] = 1, -- Hammer of Justice R3
			[10308] = 1, -- Hammer of Justice R4
			-- Priest
			[15487] = 2, -- Silence
			-- Rogue
			[1766] = 2, -- Kick R1
			[1767] = 2, -- Kick R2
			[1768] = 2, -- Kick R3
			[1769] = 2, -- Kick R4
			[38768] = 2, -- Kick R5
			--[51680] = 1, -- Throwing Specialization
			-- Shaman
			[8042] = 1, -- Earth Shock R1
			[8044] = 1, -- Earth Shock R2
			[8045] = 1, -- Earth Shock R3
			[8046] = 1, -- Earth Shock R4
			[10412] = 1, -- Earth Shock R5
			[10413] = 1, -- Earth Shock R6
			[10414] = 1, -- Earth Shock R7
			[25454] = 1, -- Earth Shock R8
			[49230] = 1, -- Earth Shock R9
			[49231] = 1, -- Earth Shock R10
			[57994] = 1, -- Wind Shock
			-- Warlock
			[19244] = 2, -- Spell Lock R1
			[19647] = 2, -- Spell Lock R2
			-- Warrior
			[6552] = 2, -- Pummel R1
			[6554] = 2, -- Pummel R2
			[72] = 2, -- Shield Bash R1
			[1671] = 2, -- Shield Bash R2
			[1672] = 2, -- Shield Bash R3
			[29704] = 2, -- Shield Bash R4
		}
    },
}

-- Creates a checkbox table element for AceConfig
-- @param One or several SpellIDs
function IEInterruptAnnounce:CreateSpellOptionElement(...)
	local firstSpellID = ...
	local allArgs = {...}
	local tempVar = {
		type = "toggle",
		tristate = true,
		name = function() return IEInterruptAnnounce:GetSpellName(firstSpellID) end,
		desc = L["|cffffff33Checked|cffffffff - Announcing\n|cffccccccGray|cffffffff - Announcing but not wasted\n|cffff0000Unchecked|cffffffff - Not announcing"],
		set = function(info,val) IEInterruptAnnounce:SetSpellSetting(val,unpack(allArgs)) end,
		get = function(info) return IEInterruptAnnounce:GetSpellSetting(firstSpellID) end
	}
	return tempVar
end

function IEInterruptAnnounce:ClassMismatchCheck(compareClass)
	local _, playerClass = UnitClass("player")
	if playerClass == compareClass then
		return false
	else
		return true
	end
end

function IEInterruptAnnounce:RaceMismatchCheck(compareRace)
	local _, playerRace = UnitRace("player")
	if playerRace == compareRace then
		return false
	else
		return true
	end
end

function IEInterruptAnnounce:IsNPCEvent(flags)
	return bit.band(flags, 0x0000FC) == 0x000008
end

local options = {
    name = "IEInterruptAnnounce",
    handler = IEInterruptAnnounce,
    type = 'group',
    args = {
		explanation = {
			type = "description",
			name = L["Tracks and announces interrupts (eg. Hit/Miss/Immune) and is also able to track and report 'wasted' interrupts (those which did not actually interrupt anything)"],
		},
		announce = {
			type = "group",
			name = L["Announce"],
			desc = L["The channels we want to announce to."],
			childGroups = "select",
			order = 100,
			args = {
				["1"] = {
					type = "group",
					name = L["Solo"],
					desc = L["The channels we want to announce to while not grouped."],
					order = 100,
					args = {
						explanation = {
							type = "description",
							order = 90,
							name = L["The channels we want to announce to while not grouped."],
						},
						say = {
							type = "toggle",
							name = L["Say"],
							width = "full",
							order = 100,
							set = function(info,val) IEInterruptAnnounce.db.profile.solo.say = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.solo.say end
						},
						party = {
							type = "toggle",
							name = L["Party"],
							width = "full",
							order = 200,
							disabled = true,
						},
						raid = {
							type = "toggle",
							name = L["Raid"],
							width = "full",
							order = 300,
							disabled = true,
						},
						raidwarning = {
							type = "toggle",
							name = L["RaidWarning"],
							width = "full",
							order = 400,
							disabled = true,
						},
						whisper = {
							type = "toggle",
							name = L["Whisper"],
							width = "full",
							order = 500,
							set = function(info,val) IEInterruptAnnounce.db.profile.solo.whisper = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.solo.whisper end
						},
						channel = {
							type = "toggle",
							name = L["Channel"],
							width = "full",
							order = 600,
							set = function(info,val) IEInterruptAnnounce.db.profile.solo.channel = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.solo.channel end
						},
					},
				},
				["2"] = {
					type = "group",
					name = L["Party"],
					desc = L["The channels we want to announce to while grouped, but not in raid."],
					order = 200,
					args = {
						explanation = {
							type = "description",
							order = 90,
							name = L["The channels we want to announce to while grouped, but not in raid."],
						},
						say = {
							type = "toggle",
							name = L["Say"],
							width = "full",
							order = 100,
							set = function(info,val) IEInterruptAnnounce.db.profile.party.say = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.party.say end
						},
						party = {
							type = "toggle",
							name = L["Party"],
							width = "full",
							order = 200,
							set = function(info,val) IEInterruptAnnounce.db.profile.party.party = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.party.party end
						},
						raid = {
							type = "toggle",
							name = L["Raid"],
							width = "full",
							order = 300,
							disabled = true,
						},
						raidwarning = {
							type = "toggle",
							name = L["RaidWarning"],
							width = "full",
							order = 400,
							disabled = true,
						},
						whisper = {
							type = "toggle",
							name = L["Whisper"],
							width = "full",
							order = 500,
							set = function(info,val) IEInterruptAnnounce.db.profile.party.whisper = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.party.whisper end
						},
						channel = {
							type = "toggle",
							name = L["Channel"],
							width = "full",
							order = 600,
							set = function(info,val) IEInterruptAnnounce.db.profile.party.channel = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.party.channel end
						},
					},
				},
				["3"] = {
					type = "group",
					name = L["Raid"],
					desc = L["The channels we want to announce to while in a raid."],
					order = 300,
					args = {
						explanation = {
							type = "description",
							order = 90,
							name = L["The channels we want to announce to while in a raid."],
						},
						say = {
							type = "toggle",
							name = L["Say"],
							width = "full",
							order = 100,
							set = function(info,val) IEInterruptAnnounce.db.profile.raid.say = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.raid.say end
						},
						party = {
							type = "toggle",
							name = L["Party"],
							width = "full",
							order = 200,
							set = function(info,val) IEInterruptAnnounce.db.profile.raid.party = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.raid.party end
						},
						raid = {
							type = "toggle",
							name = L["Raid"],
							width = "full",
							order = 300,
							set = function(info,val) IEInterruptAnnounce.db.profile.raid.raid = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.raid.raid end
						},
						raidwarning = {
							type = "toggle",
							name = L["RaidWarning"],
							width = "full",
							order = 400,
							set = function(info,val) IEInterruptAnnounce.db.profile.raid.raidwarning = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.raid.raidwarning end
						},
						whisper = {
							type = "toggle",
							name = L["Whisper"],
							width = "full",
							order = 500,
							set = function(info,val) IEInterruptAnnounce.db.profile.raid.whisper = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.raid.whisper end
						},
						channel = {
							type = "toggle",
							name = L["Channel"],
							width = "full",
							order = 600,
							set = function(info,val) IEInterruptAnnounce.db.profile.raid.channel = val end,
							get = function(info) return IEInterruptAnnounce.db.profile.raid.channel end
						},
					},
				},
				whisperTo = {
					type = "input",
					name = L["Whisper To"],
					desc = L["The name of the player you want to announce to."],
					usage = L["<Name>"],
					order = 400,
					set = function(info,val) IEInterruptAnnounce.db.profile.whisperName = val end,
					get = function(info) return IEInterruptAnnounce.db.profile.whisperName end
				},
				channelName = {
					type = "input",
					name = L["Channel Name"],
					desc = L["The name of the channel you want to announce to."],
					usage = L["<Channel Name>"],
					order = 500,
					set = function(info,val) IEInterruptAnnounce.db.profile.channelName = val end,
					get = function(info) return IEInterruptAnnounce.db.profile.channelName end
				},
			},
		},
		spells = {
			type = "group",
			name = L["Spells"],
			desc = L["The spells that should be announced."],
			order = 200,
			childGroups = "tab",
			args = {
				explanation = {
					type = "description",
					name = L["|cffffff33Checked|cffffffff - Announcing\n|cffccccccGray|cffffffff - Announcing but not wasted\n|cffff0000Unchecked|cffffffff - Not announcing"],
				},
				bloodelf = {
					type = "group",
					name = L["Blood Elf"],
					order = 90,
					hidden = IEInterruptAnnounce:RaceMismatchCheck("BloodElf"),
					args = {
						arcanetorrent = IEInterruptAnnounce:CreateSpellOptionElement(50613,28730,25046),
					},
				},
				deathknight = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["DEATHKNIGHT"],
					order = 100,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("DEATHKNIGHT"),
					args = {
						mindfreeze = IEInterruptAnnounce:CreateSpellOptionElement(47528),
						strangulate = IEInterruptAnnounce:CreateSpellOptionElement(47476),
					},
				},
				druid = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["DRUID"],
					order = 200,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("DRUID"),
					args = {
						bash = IEInterruptAnnounce:CreateSpellOptionElement(8983, 6798, 5211),
						feralcharge = IEInterruptAnnounce:CreateSpellOptionElement(19675),
					},
				},
				hunter = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["HUNTER"],
					order = 300,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("HUNTER"),
					args = {
						silencingshot = IEInterruptAnnounce:CreateSpellOptionElement(34490),
					},
				},
				mage = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["MAGE"],
					order = 400,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("MAGE"),
					args = {
						counterspell = IEInterruptAnnounce:CreateSpellOptionElement(2139),
					},
				},
				paladin = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["PALADIN"],
					order = 500,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("PALADIN"),
					args = {
						hammerofjustice = IEInterruptAnnounce:CreateSpellOptionElement(10308,5589,5588,853),
					},
				},
				priest = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["PRIEST"],
					order = 600,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("PRIEST"),
					args = {
						silence = IEInterruptAnnounce:CreateSpellOptionElement(15487),
					},
				},
				rogue = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["ROGUE"],
					order = 700,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("ROGUE"),
					args = {
						kick = IEInterruptAnnounce:CreateSpellOptionElement(38768,1769,1768,1767,1766),
						-- throwingspecialization = IEInterruptAnnounce:CreateSpellOptionElement(51680),
					},
				},
				shaman = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["SHAMAN"],
					order = 800,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("SHAMAN"),
					args = {
						earthshock = IEInterruptAnnounce:CreateSpellOptionElement(49231,49230,25454,10414,10413,10412,8046,8045,8044,8042),
						windshock = IEInterruptAnnounce:CreateSpellOptionElement(57994),
					},
				},
				warlock = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["WARLOCK"],
					order = 900,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("WARLOCK"),
					args = {
						spelllock = IEInterruptAnnounce:CreateSpellOptionElement(19647,19244),
					},
				},
				warrior = {
					type = "group",
					name = LOCALIZED_CLASS_NAMES_MALE["WARRIOR"],
					order = 1000,
					hidden = IEInterruptAnnounce:ClassMismatchCheck("WARRIOR"),
					args = {
						pummel = IEInterruptAnnounce:CreateSpellOptionElement(6554,6552),
						shieldbash = IEInterruptAnnounce:CreateSpellOptionElement(29704,1672,1671,72),
					},
				},
			},
		},
		advanced = {
			type = "group",
			name = L["Advanced"],
			desc = L["Advanced settings."],
			order = 300,
			args = {
				debug = {
					type = "toggle",
					name = L["Debug"],
					order = 100,
					width = "full",
					set = function(info,val) IEInterruptAnnounce.db.profile.advanced.debug = val end,
					get = function(info) return IEInterruptAnnounce.db.profile.advanced.debug end
				},
				discoverInterruptMode = {
					type = "toggle",
					name = L["Discover Interrupt Mode"],
					desc = L["If an interrupt occurs and it is not in the spelllist table, a local message is sent."],
					order = 200,
					width = "full",
					set = function(info,val) IEInterruptAnnounce.db.profile.advanced.discoverInterruptMode = val end,
					get = function(info) return IEInterruptAnnounce.db.profile.advanced.discoverInterruptMode end
				},
				chatThrottle = {
					type = "input",
					name = L["Chat Throttle Time"],
					order = 300,
					desc = L["The minimum amount of time between chat messages."],
					pattern = "^%d+$",
					usage = L["<Time In Milliseconds>"],
					set = function(info,val) IEInterruptAnnounce.db.profile.advanced.chatThrottleMillis = tonumber(val) end,
					get = function(info) return tostring(IEInterruptAnnounce.db.profile.advanced.chatThrottleMillis) end
				},
				intDelay = {
					type = "input",
					name = L["Interrupt Delay"],
					order = 400,
					desc = L["The amount of time to wait for the interrupt after the spell was cast, before discarding the interrupt as wasted."],
					pattern = "^%d+$",
					usage = L["<Time In Milliseconds>"],
					set = function(info,val) IEInterruptAnnounce.db.profile.advanced.interruptDelayMillis = tonumber(val) end,
					get = function(info) return tostring(IEInterruptAnnounce.db.profile.advanced.interruptDelayMillis) end
				},
				msgFormat = {
					type = "input",
					name = L["Message Format"],
					order = 500,
					desc = L["The format of the announce messages, should contain 3 instances of %s"],
					pattern = "^.*%%s.*%%s.*%%s.*$",
					usage = L["ex. %s: %s (%s)"],
					set = function(info,val) IEInterruptAnnounce.db.profile.advanced.msgFormat = val end,
					get = function(info) return IEInterruptAnnounce.db.profile.advanced.msgFormat end
				},
				wastedText = {
					type = "input",
					name = L["Wasted Text"],
					order = 600,
					desc = L["The text that should be shown instead of the interrupted spell name when an interrupt is used without actually interrupting."],
					usage = L["ex. <Wasted>"],
					set = function(info,val) IEInterruptAnnounce.db.profile.advanced.wastedText = val end,
					get = function(info) return IEInterruptAnnounce.db.profile.advanced.wastedText end
				},
			},
		},
    },
}

function IEInterruptAnnounce:OnInitialize()
    -- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("IEInterruptAnnounceDB", defaults, "Default")

	LibStub("AceConfig-3.0"):RegisterOptionsTable("IEInterruptAnnounce", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("IEInterruptAnnounce", "IEInterruptAnnounce")
	self:RegisterChatCommand("ieinterruptannounce", "ChatCommand")
    self:RegisterChatCommand("ieia", "ChatCommand")
end

-- Sets the same value to several SpellIDs, recursively
function IEInterruptAnnounce:SetSpellSetting(value, spellID, ...)
	-- Translates the value from the checkbox to the spell table
	if value == true then
		value = 2
	elseif value == nil then
		value = 1
	elseif value == false then
		value = 0
	end
	
	if spellID ~= nil then
		self.db.profile.spellList[spellID] = value
		self:SetSpellSetting(value, ...)
	end
end

function IEInterruptAnnounce:GetSpellSetting(spellID)
	if IEInterruptAnnounce:isTrackedSpell(spellID) then
		return true
	elseif IEInterruptAnnounce:isAnnounceSpell(spellID) then
		return nil
	else
		return false
	end
end

function IEInterruptAnnounce:isAnnounceSpell(spellID)
	return (self.db.profile.spellList[spellID] and self.db.profile.spellList[spellID] >= 1)
end

function IEInterruptAnnounce:isTrackedSpell(spellID)
	return (self.db.profile.spellList[spellID] and self.db.profile.spellList[spellID] == 2)
end

function IEInterruptAnnounce:GetSpellName(spellID)
	name = GetSpellInfo(spellID)
	return name
end

function IEInterruptAnnounce:OnEnable()
    -- Called when the addon is enabled
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function IEInterruptAnnounce:OnDisable()
    -- Called when the addon is disabled
end

function IEInterruptAnnounce:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(IEInterruptAnnounce, "ieia", "IEInterruptAnnounce", input)
    end
end

function IEInterruptAnnounce:sendMessage(...)
	if cooldownReadyTime < GetTime() then
		cooldownReadyTime = GetTime()+(self.db.profile.advanced.chatThrottleMillis/1000);
		msg = IEInterruptAnnounce.db.profile.advanced.msgFormat:format(...)
		
		-- What profile to use? Solo / Party / Raid
		local announceSettings
		if inRaid then
			announceSettings = IEInterruptAnnounce.db.profile.raid
		elseif grouped then
			announceSettings = IEInterruptAnnounce.db.profile.party
		else
			announceSettings = IEInterruptAnnounce.db.profile.solo
		end
		
		if announceSettings.say == true then
			SendChatMessage(msg, "SAY")
		end
		if announceSettings.whisper == true then
			SendChatMessage(msg, "WHISPER", nil, IEInterruptAnnounce.db.profile.whisperName)
		end
		if announceSettings.channel == true then
			local chanNr = GetChannelName(IEInterruptAnnounce.db.profile.channelName);
			if (chanNr~=nil) then -- You are in the channel
				SendChatMessage(msg, "CHANNEL", nil, chanNr)
			end
		end
		
		if grouped and (not inBG) then
			if announceSettings.party == true then
				SendChatMessage(msg, "PARTY")
			end
			if inRaid then
				if announceSettings.raid == true and inRaid == true then
					SendChatMessage(msg, "RAID")
				end
				if announceSettings.raidwarning == true and inRaid == true and IsRaidOfficer() then
					SendChatMessage(msg, "RAID_WARNING")
				end
			end
		end
	elseif IEInterruptAnnounce.db.profile.advanced.debug == true then
		self:Print("Threw away message. - "..IEInterruptAnnounce.db.profile.advanced.msgFormat:format(...))
	end
end

function IEInterruptAnnounce:PLAYER_ENTERING_WORLD()
	local inInstance, instanceType = IsInInstance()
	-- inInstance 
	-- 	Boolean - Whether the player is in an instance; nil otherwise. 
	-- instanceType 
	-- 	String - The instance type: 
    --    * "none" when outside an instance
    --    * "pvp" when in a battleground
    --    * "arena" when in an arena
    --    * "party" when in a 5-man instance
    --    * "raid" when in a raid instance
	
	if instanceType == "pvp" then
		inBG = true
		inArena = false
	elseif instanceType == "arena" then
		inBG = false
		inArena = true
	end
end

function IEInterruptAnnounce:PARTY_MEMBERS_CHANGED()
	local n = GetNumRaidMembers()
	if n == 0 then -- Not in raid
		inRaid = false
		n = GetNumPartyMembers()
		if n > 0 then -- In party, but not raid
			grouped = true
		else -- Not in raid, nor party
			grouped = false
		end
	else -- In raid, and therefore also grouped
		inRaid = true
		grouped = true
	end
end

function IEInterruptAnnounce:COMBAT_LOG_EVENT_UNFILTERED(_,timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, ...)
	if destName == nil then
		destName = "No Target"
	end
	-- Combat log event
	if interruptTimestamp ~= nil then -- An interrupting spell has been used without invoking an interrupt, yet
		local timeSinceInterrupt = (timestamp - interruptTimestamp)*1000 -- Milliseconds
		if (timeSinceInterrupt  >= IEInterruptAnnounce.db.profile.advanced.interruptDelayMillis) then -- The interrupt isn't coming, the kick was wasted
			interruptTimestamp = nil -- Stop tracking kicks already made
			IEInterruptAnnounce:sendMessage(unpack(interruptVariables))
		end
	end
	
	if event == "SPELL_INTERRUPT" then -- A successful interrupt'
		local spellID, spellName, _, _, interruptedSpellName = ...
		if IEInterruptAnnounce:isAnnounceSpell(spellID) == true then -- A whitelisted interrupt was used
			if (sourceGUID == UnitGUID("player")) or (bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) > 0) then -- The interrupt is yours
				interruptTimestamp = nil -- Stop tracking kicks already made
				IEInterruptAnnounce:sendMessage(spellName, interruptedSpellName, destName)
			end
		elseif (IEInterruptAnnounce.db.profile.advanced.discoverInterruptMode == true) and defaults.profile.spellList[spellID] == nil then
			if (not IEInterruptAnnounce:IsNPCEvent(sourceFlags)) then
				self:Print(sourceName.."'s "..GetSpellLink(spellID).." ["..spellID.."] interrupted "..interruptedSpellName..".")
			end
		end
	elseif event == "SPELL_CAST_SUCCESS" then
		local spellID, spellName, spellSchool = ...
		if IEInterruptAnnounce:GetSpellSetting(spellID) == true then -- A whitelisted interrupt (with waste tracking) was used
			if (sourceGUID == UnitGUID("player")) or (bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) > 0) then -- The interrupt is yours
				interruptTimestamp = timestamp
				interruptVariables = {spellName, IEInterruptAnnounce.db.profile.advanced.wastedText, destName}
				if IEInterruptAnnounce.db.profile.advanced.debug == true then
					self:Print(spellName.." on "..destName)
				end
			end
		end
	elseif eventType == "SPELL_MISSED" then
		local spellID, spellName, spellSchool, missType = ...
		if IEInterruptAnnounce:isAnnounceSpell(spellID) == true then -- A whitelisted interrupt was used
			if (sourceGUID == UnitGUID("player")) or (bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) > 0) then -- The interrupt is yours
				interruptTimestamp = nil -- Stop tracking interrupts already made
				IEInterruptAnnounce:sendMessage(spellName, missType, destName)
			end
		end
	end
end
